# 底盘与工控机之间的CAN通信

## 依赖

ros包目前的搭建环境为Ubuntu18.04LTS

## 硬件连接

将工控机 CAN-H 与Viper-A的 CAN-H 连接, CAN-L 与 CAN-L 连接。

## 配置

在根目录下开启终端，创建工作空间（工作空间默认位置为根目录，默认名称为mini_chassis）。

```bash
mkdir mini_chassis
mkdir mini_chassis/src
```

将`socketcan_bridge`和`socketcan_interface`文件放置在工作空间的`/src`目录下，回到`mini_chassis`目录下编译。can0配置文件` can.sh `可以放在任意位置。

```bash
cd ~/mini_chassis
catkin build
```

## 配置can0

### 脚本一键配置

在每次开机后都需要对can0进行配置，在脚本` can.sh `的存放目录下开启一个终端，输入` ./can.sh `即可启动脚本，完成can0的快速配置并开启can0。返回结果大致如下：

```shell
can0: flags=193<UP,RUNNING,NOARP>  mtu 16
        unspec 00-00-00-00-00-00-00-00-00-00-00-00-00-00-00-00  txqueuelen 10  (未指定)
        RX packets 0  bytes 0 (0.0 B)
        RX errors 0  dropped 0  overruns 0  frame 0
        TX packets 0  bytes 0 (0.0 B)
        TX errors 0  dropped 0 overruns 0  carrier 0  collisions 0
        device interrupt 64
```

### 开机自启

检查工控机中是否有`/etc/rc.local`文件，如果不存在，在终端中执行以下命令创建它：

```bash
sudo touch /etc/rc.local
sudo chmod +x /etc/rc.local
```

将can.sh文件放入/etc中
执行`sudo vim /etc/rc.local`打开`rc.local`文件并编辑：

```bash
#! /bin/bash
sh /etc/can.sh &
exit 0
```

至此，can0口开机自启配置完成。
查看can0口状态`ifconfig can0`

## 启动

开启一个新终端，执行：

```bash
cd mini_chassis
source ./devel/setup.bash
roslaunch socketcan_bridge deploy.launch
```

此时会开启多个话题，以下是各话题相关的内容：

话题名：`/change_mode`  
话题类型：`std_msgs.msg/Int32`  
话题说明：用来切换底盘的运动模式和重置里程计数据，底盘默认的运动模式为Ackermann（阿克曼模式）。
使用方法：在`~/mini_chassis/src/socketcan_bridge/src`目录下打开终端，输入`python xxx.py`运行。其中，`xxx`可以为：  
`park`          切换为驻车模式  
`spin`          切换为自旋模式  
`diagonal`      切换为斜移模式  
`ackermann`     切换为阿克曼模式  
`disabled`      切换为失能模式  
(失能模式下，轮子会维持前一个模式的状态，但无法进行控制。想要恢复控制，需要先切换为其他模式)  
`reset_odom`    重置底盘的里程计数据  
运行后等待2秒左右，完成相关指令，python程序会自动关闭。

话题名：`/chassis_state`  
话题类型：`socketcan_bridge/ViperCanData.h`  
话题说明：反馈底盘的运动模式、剩余电量、速度和充电状态等信息。
话题变量：  
        `motion_mode`           底盘当前运动模式  
        `remaining_battery`     底盘当前的剩余电量百分比  
        `status`                `charge`        充电  
                                `discharge`     未充电
                                `Stop`          急停按钮按下  
                                `Running`       急停按钮弹起

话题名：`/chassis_odom`  
话题类型：`nav_msgs/Odometry.h`  
话题说明：反馈底盘的里程计数据。  
话题变量：  
        `pose.pose.position x`          底盘相对于里程计原点的x轴位移  
        `pose.pose.position y`          底盘相对于里程计原点的y轴位移  
        `twist.twist.linear x`          底盘当前整车线速度，单位：m/s  
        `twist.twist.angular z`         底盘当前整车角速度，单位：rad/s  

话题名：`/chassis_imu`  
话题类型：`sensor_msgs/Imu.h`  
话题说明：反馈底盘的IMU数据。  
话题变量：  
        `angular_velocity`              角速度，单位：rad/s
        `linear_acceleration.x`         底盘在x轴方向的加速度  
        `linear_acceleration.y`         底盘在y轴方向的加速度  
        `linear_acceleration.z`         底盘在z轴方向的加速度（重力加速度）

话题名：`/chassis_current_and_temperature_of_left_hub_motor`  
话题类型：`socketcan_bridge/current_and_temperature_of_left_hub_motor.h`  
话题说明：反馈底盘左侧轮毂电机的电流和温度。  
话题变量：  
        `left_front_hub_motor_current`          底盘1号左前轮毂电机的母线电流，单位：A
        `left_front_hub_motor_temperature`      底盘1号左前轮毂电机的温度，单位：°C
        `left_back_hub_motor_current`           底盘2号左后轮毂电机的母线电流，单位：A
        `left_back_hub_motor_temperature`       底盘2号左后轮毂电机的温度，单位：°C

话题名：`/chassis_current_and_temperature_of_right_hub_motor`  
话题类型：`socketcan_bridge/current_and_temperature_of_right_hub_motor.h`  
话题说明：反馈底盘右侧轮毂电机的电流和温度。  
话题变量：  
        `right_front_hub_motor_current`          底盘4号右前轮毂电机的母线电流，单位：A
        `right_front_hub_motor_temperature`      底盘4号右前轮毂电机的温度，单位：°C
        `right_back_hub_motor_current`           底盘3号右后轮毂电机的母线电流，单位：A
        `right_back_hub_motor_temperature`       底盘3号右后轮毂电机的温度，单位：°C

话题名：`/chassis_velocity_and_angle_of_left_hub_motor_and_steering_engine`  
话题类型：`socketcan_bridge/velocity_and_angle_of_left_hub_motor_and_steering_engine.h`  
话题说明：反馈底盘左侧轮毂电机的转速和舵机角度。  
话题变量：  
        `left_front_hub_motor_velocity`          底盘1号左前轮毂电机转速，单位：°/s
        `left_front_steering_engine_angle`       底盘1号左前舵机角度，单位：°
        `left_back_hub_motor_velocity`           底盘2号左后轮毂电机转速，单位：°/s
        `left_back_steering_engine_angle`        底盘2号左后舵机角度，单位：°

话题名：`/chassis_velocity_and_angle_of_right_hub_motor_and_steering_engine`  
话题类型：`socketcan_bridge/velocity_and_angle_of_right_hub_motor_and_steering_engine.h`  
话题说明：反馈底盘右侧轮毂电机的转速和舵机角度。  
话题变量：  
        `right_front_hub_motor_velocity`          底盘4号右前轮毂电机转速，单位：°/s
        `right_front_steering_engine_angle`       底盘4号右前舵机角度，单位：°
        `right_back_hub_motor_velocity`           底盘3号右后轮毂电机转速，单位：°/s
        `right_back_steering_engine_angle`        底盘3号右后舵机角度，单位：°

话题名：`/chassis_voltage_and_temperate_of_control_card`  
话题类型：`socketcan_bridge/voltage_and_temperate_of_control_card.h`  
话题说明：反馈底盘电机驱动板的电压和温度。  
话题变量：  
        `left_board_voltage`                      底盘左电机驱动板的电压采样，单位：V
        `left_board_mos_temperature`              底盘左电机驱动板的mos温度采样，单位：°C
        `right_board_voltage`                     底盘右电机驱动板的电压采样，单位：V
        `right_board_mos_temperature`             底盘右电机驱动板的mos温度采样，单位：°C

话题名：`/chassis_state_of_steering_engine_and_hub_motor`  
话题类型：`socketcan_bridge/state_of_steering_engine_and_hub_motor.h`  
话题说明：反馈底盘各轮毂电机和舵机的状态。  
话题变量：  
        `state_of_steering_engine_and_hub_motor`  底盘轮毂电机或舵机是否出现失能
说明：1.此处的“舵机到达”不等于完全到达，而是离目标角度还有一个小的角度范围认为到达，到达后下发轮毂电机速度指令。
     2.当所有的轮毂电机和舵机都正常使能时，话题输出内容为空。当某一轮毂电机或舵机未使能时，话题输出为：`state_of_steering_engine_and_hub_motor = xxx disabled`。

同时，底盘开始订阅`/cmd_vel`话题以获取运动控制指令。

## 备注

如果在使用过程中出现任一ros节点闪退的情况，请关闭并重新启动`deploy.launch`。  

关于整车线速度和角速度的一些说明：  

1. Ackermann模式下直线行驶时，整车线速度的上限约为1.3m/s。
2. Ackermann模式下转向行驶时，整车线速度上限约为1.0m/s，角速度上限约为2.2rad/s。
3. Spin模式或Ackermann模式只提供底盘角速度时，整车角速度上限约为3.14rad/s。

调试仍在进行中，ros包及文档会持续更新。
