#include <ros/ros.h>
#include <can_msgs/Frame.h>
#include <socketcan_bridge/current_and_temperature_of_left_hub_motor.h>
#include <std_msgs/String.h>
#include <iostream>

socketcan_bridge::current_and_temperature_of_left_hub_motor parse_can_data_frame(const std::vector<uint8_t> &data_frame)
{
    socketcan_bridge::current_and_temperature_of_left_hub_motor can_data;

    double a = (data_frame[0] << 8) | data_frame[1];
    double b = (data_frame[2] << 8) | data_frame[3];
    double c = (data_frame[4] << 8) | data_frame[5];
    double d = (data_frame[6] << 8) | data_frame[7];

    can_data.left_front_hub_motor_current = a / 100;
    can_data.left_front_hub_motor_temperature = b / 100;
    can_data.left_back_hub_motor_current = c / 100;
    can_data.left_back_hub_motor_temperature = d / 100;

    return can_data;
}

void received_messages_callback(const can_msgs::Frame::ConstPtr &msg, ros::Publisher &pub)
{
    if (msg->id == 277) // 如果id不是277则直接返回 return;
    {
        std::vector<uint8_t> data_frame(msg->data.begin(), msg->data.end());
        socketcan_bridge::current_and_temperature_of_left_hub_motor can_data = parse_can_data_frame(data_frame);
        pub.publish(can_data);
    }
}

int main(int argc, char **argv)
{
    ros::init(argc, argv, "current_and_temperature_of_left_hub_motor");
    ros::NodeHandle nh;

    ros::Publisher can_data_pub = nh.advertise<socketcan_bridge::current_and_temperature_of_left_hub_motor>("/chassis_current_and_temperature_of_left_hub_motor", 10);
    ros::Subscriber received_messages_sub = nh.subscribe<can_msgs::Frame>("/received_messages", 1, boost::bind(received_messages_callback, _1, can_data_pub));

    ros::spin();
    return 0;
}