#include <ros/ros.h>
#include <can_msgs/Frame.h>
#include <sensor_msgs/Imu.h>
#include <vector>

ros::Publisher imu_pub;

std::vector<can_msgs::Frame> imu_buffer; // 定义缓存区，用来暂存底盘先发送的can帧，等待后续can帧都到达后一并进行转换并发送到ros话题。

sensor_msgs::Imu can_frames_to_imu(const std::vector<can_msgs::Frame> &frames)
{
    sensor_msgs::Imu imu_msg;
    std::vector<uint8_t> data_274, data_275, data_276;
    for (const auto &frame : frames)
    {
        std::vector<uint8_t> data;
        std::copy(frame.data.begin(), frame.data.end(), std::back_inserter(data));
        if (frame.id == 274)
        {
            data_274 = data;
        }
        else if (frame.id == 275)
        {
            data_275 = data;
        }
        else if (frame.id == 276)
        {
            data_276 = data;
        }
    }

    float accx, accy, accz, wz;
    unsigned char temp[4];
    temp[0] = data_275[3];
    temp[1] = data_275[2];
    temp[2] = data_275[1];
    temp[3] = data_275[0];
    std::memcpy(&accx, temp, sizeof(float));
    temp[0] = data_275[7];
    temp[1] = data_275[6];
    temp[2] = data_275[5];
    temp[3] = data_275[4];
    std::memcpy(&accy, temp, sizeof(float));
    temp[0] = data_276[3];
    temp[1] = data_276[2];
    temp[2] = data_276[1];
    temp[3] = data_276[0];
    std::memcpy(&accz, temp, sizeof(float));
    temp[0] = data_274[7];
    temp[1] = data_274[6];
    temp[2] = data_274[5];
    temp[3] = data_274[4];
    std::memcpy(&wz, temp, sizeof(float));

    imu_msg.header.stamp = ros::Time::now();
    imu_msg.header.frame_id = "chassis_imu";
    imu_msg.linear_acceleration.x = accx;
    imu_msg.linear_acceleration.y = accy;
    imu_msg.linear_acceleration.z = accz;
    imu_msg.angular_velocity.z = wz;

    return imu_msg;
}

void received_messages_callback(const can_msgs::Frame::ConstPtr &msg)
{
    // 将接收到的can帧存储到缓存区
    imu_buffer.push_back(*msg);

    bool found_274 = false, found_275 = false, found_276 = false;
    for (const auto &frame : imu_buffer)
    {
        if (frame.id == 274)
        {
            found_274 = true;
        }
        else if (frame.id == 275)
        {
            found_275 = true;
        }
        else if (frame.id == 276)
        {
            found_276 = true;
        }
    }

    if (found_274 && found_275 && found_276)
    {
        sensor_msgs::Imu imu_msg = can_frames_to_imu(imu_buffer);

        std::cout << "The IMU Data of small chassis: \n";
        std::cout << imu_msg;
        imu_pub.publish(imu_msg);

        // 清空缓存区
        imu_buffer.clear();
    }
}

int main(int argc, char **argv)
{
    ros::init(argc, argv, "chassis_imu_node");
    ros::NodeHandle nh;

    imu_pub = nh.advertise<sensor_msgs::Imu>("/chassis_imu", 10);
    ros::Subscriber can_sub = nh.subscribe<can_msgs::Frame>("/received_messages", 10, received_messages_callback);

    ros::spin();

    return 0;
}