#include <ros/ros.h>
#include <can_msgs/Frame.h>
#include <socketcan_bridge/state_of_steering_engine_and_hub_motor.h>
#include <std_msgs/String.h>
#include <iostream>

socketcan_bridge::state_of_steering_engine_and_hub_motor parse_can_data_frame(const std::vector<uint8_t> &data_frame)
{
    if (data_frame.size() != 8)
        throw std::invalid_argument("Invalid data frame length"); // 如果收到的can数据帧的长度不等于8就报异常。

    socketcan_bridge::state_of_steering_engine_and_hub_motor can_data;
    unsigned char hex = data_frame[0];
    std::string state;

    if (!(hex & 0x80))
    {
        state += "left_front_steering_engine disabled. ";
    }
    if (!(hex & 0x40))
    {
        state += "left_back_steering_engine disabled. ";
    }
    if (!(hex & 0x20))
    {
        state += "right_back_steering_engine disabled. ";
    }
    if (!(hex & 0x10))
    {
        state += "right_front_steering_engine disabled. ";
    }
    if (!(hex & 0x08))
    {
        state += "left_front_hub_motor disabled. ";
    }
    if (!(hex & 0x04))
    {
        state += "left_back_hub_motor disabled. ";
    }
    if (!(hex & 0x02))
    {
        state += "right_back_hub_motor disabled. ";
    }
    if (!(hex & 0x01))
    {
        state += "right_front_hub_motor disabled. ";
    }
    can_data.state_of_steering_engine_and_hub_motor = state;
    return can_data;
}

void received_messages_callback(const can_msgs::Frame::ConstPtr &msg, ros::Publisher &pub)
{
    if (msg->id == 282) // 如果id不是272则直接返回 return;
    {
        std::vector<uint8_t> data_frame(msg->data.begin(), msg->data.end());                                  // 将接收到的数据转化为字节向量。
        socketcan_bridge::state_of_steering_engine_and_hub_motor can_data = parse_can_data_frame(data_frame); // 将字节向量转化成自定义消息类型state_of_steering_engine_and_hub_motor。

        std::cout << "The motion state parameters of small chassis: \n";
        std::cout << can_data;

        pub.publish(can_data);
    }
}

int main(int argc, char **argv)
{
    ros::init(argc, argv, "state_of_steering_engine_and_hub_motor");
    ros::NodeHandle nh;

    ros::Publisher can_data_pub = nh.advertise<socketcan_bridge::state_of_steering_engine_and_hub_motor>("/chassis_state_of_steering_engine_and_hub_motor", 10);
    ros::Subscriber received_messages_sub = nh.subscribe<can_msgs::Frame>("/received_messages", 1, boost::bind(received_messages_callback, _1, can_data_pub));

    ros::spin();
    return 0;
}