#include <ros/ros.h>
#include <can_msgs/Frame.h>
#include <socketcan_bridge/velocity_and_angle_of_left_hub_motor_and_steering_engine.h>
#include <std_msgs/String.h>
#include <iostream>

socketcan_bridge::velocity_and_angle_of_left_hub_motor_and_steering_engine parse_can_data_frame(const std::vector<uint8_t> &data_frame)
{
    socketcan_bridge::velocity_and_angle_of_left_hub_motor_and_steering_engine can_data;

    int16_t a = (data_frame[0] << 8) | data_frame[1];
    int16_t b = (data_frame[2] << 8) | data_frame[3];
    int16_t c = (data_frame[4] << 8) | data_frame[5];
    int16_t d = (data_frame[6] << 8) | data_frame[7];

    can_data.left_front_hub_motor_velocity = static_cast<double>(a) / 70;
    can_data.left_front_steering_engine_angle = static_cast<double>(b) / 10;
    can_data.left_back_hub_motor_velocity = static_cast<double>(c) / 70;
    can_data.left_back_steering_engine_angle = static_cast<double>(d) / 10;

    return can_data;
}

void received_messages_callback(const can_msgs::Frame::ConstPtr &msg, ros::Publisher &pub)
{
    if (msg->id == 279) // 如果id不是277则直接返回 return;
    {
        std::vector<uint8_t> data_frame(msg->data.begin(), msg->data.end());
        socketcan_bridge::velocity_and_angle_of_left_hub_motor_and_steering_engine can_data = parse_can_data_frame(data_frame);
        pub.publish(can_data);
    }
}

int main(int argc, char **argv)
{
    ros::init(argc, argv, "velocity_and_angle_of_left_hub_motor_and_steering_engine");
    ros::NodeHandle nh;

    ros::Publisher can_data_pub = nh.advertise<socketcan_bridge::velocity_and_angle_of_left_hub_motor_and_steering_engine>("/chassis_velocity_and_angle_of_left_hub_motor_and_steering_engine", 10);
    ros::Subscriber received_messages_sub = nh.subscribe<can_msgs::Frame>("/received_messages", 1, boost::bind(received_messages_callback, _1, can_data_pub));

    ros::spin();
    return 0;
}