#include <ros/ros.h>
#include <can_msgs/Frame.h>
#include <socketcan_bridge/voltage_and_temperate_of_control_card.h>
#include <std_msgs/String.h>
#include <iostream>

socketcan_bridge::voltage_and_temperate_of_control_card parse_can_data_frame(const std::vector<uint8_t> &data_frame)
{
    socketcan_bridge::voltage_and_temperate_of_control_card can_data;

    double a = (data_frame[0] << 8) | data_frame[1];
    double b = (data_frame[2] << 8) | data_frame[3];
    double c = (data_frame[4] << 8) | data_frame[5];
    double d = (data_frame[6] << 8) | data_frame[7];

    can_data.left_board_voltage = a / 100;
    can_data.left_board_mos_temperature = b / 100;
    can_data.right_board_voltage = c / 100;
    can_data.right_board_mos_temperature = d / 100;

    return can_data;
}

void received_messages_callback(const can_msgs::Frame::ConstPtr &msg, ros::Publisher &pub)
{
    if (msg->id == 281) // 如果id不是281则直接返回 return;
    {
        std::vector<uint8_t> data_frame(msg->data.begin(), msg->data.end());
        socketcan_bridge::voltage_and_temperate_of_control_card can_data = parse_can_data_frame(data_frame);
        pub.publish(can_data);
    }
}

int main(int argc, char **argv)
{
    ros::init(argc, argv, "voltage_and_temperate_of_control_card");
    ros::NodeHandle nh;

    ros::Publisher can_data_pub = nh.advertise<socketcan_bridge::voltage_and_temperate_of_control_card>("/chassis_voltage_and_temperate_of_control_card", 10);
    ros::Subscriber received_messages_sub = nh.subscribe<can_msgs::Frame>("/received_messages", 1, boost::bind(received_messages_callback, _1, can_data_pub));

    ros::spin();
    return 0;
}